# Core Models

## Setup

rails plugin new core_models --mountable -d sqlite3 -T --dummy-path=spec/dummy --skip-bundle

Add the following to the gemspec file:
  s.rdoc_options = ["--line-numbers", "--inline-source", "--title", "core_models", "--main", "README.rdoc"]
  s.extra_rdoc_files = ["CHANGELOG", "README.rdoc", "lib/tasks/core_tasks.rake", "lib/core_models.rb"]

AND

  s.add_development_dependency 'rcov', '~> 0.9.9'
  s.add_development_dependency 'rspec-rails', '~> 2.8.1'
  s.add_development_dependency 'cucumber-rails', '~> 1.3.0'
  s.add_development_dependency 'shoulda-matchers', '~> 1.2.0'
  s.add_development_dependency 'factory_bot_rails', '~> 1.7.0'
  s.add_development_dependency 'database_cleaner', '~> 0.7.1'
  s.add_development_dependency 'ci_reporter', '~> 1.7.0'

### Get basic gem generation working

rake build

### RSpec

bundle exec rails generate rspec:install

Tweaks for 'spec_helper.rb':
----------------------------
# This file is copied to spec/ when you run 'rails generate rspec:install'
ENV["RAILS_ENV"] ||= 'test'
require File.expand_path("../../config/environment.rb", __FILE__)
require File.expand_path("../../spec/dummy/config/environment.rb", __FILE__)
#require File.expand_path("../../test/dummy/config/environments/#{::Rails.env}.rb", __FILE__)

require 'rspec/rails'
require 'rspec/autorun'

require 'factory_bot_rails'
require 'database_cleaner'
require 'shoulda/matchers/integrations/rspec'

ENGINE_RAILS_ROOT=File.join(File.dirname(__FILE__), '../')

# Requires supporting ruby files with custom matchers and macros, etc,
# in spec/support/ and its subdirectories.
# Dir[Rails.root.join("spec/support/**/*.rb")].each {|f| require f}
Dir[File.join(ENGINE_RAILS_ROOT, 'spec/support/**/*.rb')].each {|f| require f }

RSpec.configure do |config|
  # ## Mock Framework
  #
  # If you prefer to use mocha, flexmock or RR, uncomment the appropriate line:
  #
  # config.mock_with :mocha
  # config.mock_with :flexmock
  # config.mock_with :rr
  config.mock_with :rspec

  # Remove this line if you're not using ActiveRecord or ActiveRecord fixtures
  config.fixture_path = "#{::Rails.root}/spec/fixtures"

  # If you're not using ActiveRecord, or you'd prefer not to run each of your
  # examples within a transaction, remove the following line or assign false
  # instead of true.
  config.use_transactional_fixtures = false

  # Use DatabaseCleaner to manage transactions
  config.before(:suite) do
    DatabaseCleaner.strategy = :transaction
    DatabaseCleaner.clean_with(:truncation)
  end
  config.before(:each) do
    DatabaseCleaner.start
  end
  config.after(:each) do
    DatabaseCleaner.clean
  end

  # If true, the base class of anonymous controllers will be inferred
  # automatically. This will be the default behavior in future versions of
  # rspec-rails.
  config.infer_base_class_for_anonymous_controllers = false

  # Include Engine routes (needed for Controller specs)
  config.include Core::Engine.routes.url_helpers
end

Test RSpec will run:
bundle exec rspec

### Cucumber

bundle exec rails generate cucumber:install --rspec --capybara

Tweaks for 'env.rb':
--------------------

ENV["RAILS_ENV"] ||= "test"
ENV["RAILS_ROOT"] ||= File.dirname(__FILE__) + "../../../spec/dummy"
require File.expand_path("../../../spec/dummy/config/environment.rb",  __FILE__)

require 'cucumber/rails'

--- AND ---

# include Engine routes in Cucumber world
module EngineRoutesHelper
  include Core::Engine.routes.url_helpers
end
World(EngineRoutesHelper)

Test Cucumber will run:
bundle exec cucumber

## Re-construct all databases

time (rake db:drop db:create db:migrate db:seed && rake db:drop db:create db:schema:load RAILS_ENV=test) &> spec/dummy/log/db_refresh.log
time rake ci:install_vendor_migrations db:drop db:create db:migrate db:seed && rake db:drop db:create db:schema:load RAILS_ENV=test &> spec/dummy/log/db_refresh.log

## Model definitions

### Core

bundle exec rails generate scaffold Base/Address room:string building:string address_line_1:string address_line_2:string town:string city:string postal_code:string region:string country:references telephone_number_1:string telephone_number_2:string fax_number_1:string fax_number_2:string email_1:string email_2:string email_3:string url_1:string url_2:string url_3:string owner:references
bundle exec rails generate scaffold Base/City name:string name_ascii:string region:string latitude:float longitude:float country:references
bundle exec rails generate scaffold Base/Colour name:string color_code:string
bundle exec rails generate scaffold Base/Currency iso_code:string name:string rate:float surcharge:float symbol:string symbol_as_prefix:boolean date_of_rate:datetime is_active:boolean
bundle exec rails generate scaffold Base/DistanceUnit short_name:string singular_name:string repeating_name:string length_in_metres:float
bundle exec rails generate scaffold Base/Gender name:string
bundle exec rails generate scaffold Base/MaritialStatus name:string
bundle exec rails generate scaffold Base/PaymentMethod name:string details:string
bundle exec rails generate scaffold Base/Region country:references code:string
bundle exec rails generate scaffold Base/Timespan short_name:string singular_name:string repeating_name:string length_in_days:float ical_expression:string

### Internationalisation (I18n)

bundle exec rails generate scaffold I18n/Country iso_code:string name:string
bundle exec rails generate scaffold I18n/Dialect name:string language:references
bundle exec rails generate scaffold I18n/Language iso_639_1_code:string iso_639_2_code:string english_name:string french_name:string native_name:string

bundle exec rails generate scaffold I18n/CountryCurrencies iso_4217_country_name:string iso_4217_3char_country_code:string iso_4217_2char_country_code:string iso_4217_numeric_country_code:string iso_3166_currency_name:string iso_3166_currency_code:string iso_3166_default_numeric_currency_code:string iso_3166_numeric_currency_code:string iso_3166_currency_minor_units:string

### People & Companies

bundle exec rails generate scaffold PeopleAndCompanies/BankBranch name:string sort_code:string bank:references
bundle exec rails generate scaffold PeopleAndCompanies/Bank name:string parent_company:references
bundle exec rails generate scaffold PeopleAndCompanies/BankPaymentCard name_on_card:string payment_card_number:string security_key:string valid_from:datetime valid_until:datetime client:references vendor:references
bundle exec rails generate scaffold PeopleAndCompanies/BankPaymentCardVendor name:string company:references
bundle exec rails generate scaffold PeopleAndCompanies/BusinessLanguage country:references language:references
bundle exec rails generate scaffold PeopleAndCompanies/CompanyBankAccount bank_account:references company:references
bundle exec rails generate scaffold PeopleAndCompanies/Company name:string fiscal_name:string branch_name:string registration_number:string vat_registration_number:string head_office_company:references parent_office_company:references postal_address:references registered_address:references registration_country:references telephone_number_1:string telephone_number_2:string telephone_number_3:string fax_number_1:string fax_number_2:string fax_number_3:string email_1:string email_2:string email_3:string url_1:string url_2:string url_3:string notes:text
bundle exec rails generate scaffold PeopleAndCompanies/JobTitle base_salary:decimal desctription:string name:string company:references department:references
bundle exec rails generate scaffold PeopleAndCompanies/MasterInformation company:references company_bank_account:references main_contact:references
bundle exec rails generate scaffold PeopleAndCompanies/OperatingCompany company:references country:references language:references
bundle exec rails generate scaffold PeopleAndCompanies/Person title:string forename:string middle_names:string surname:string photo:references home_address:references postal_address:references contact_address:references birthdate:datetime maritial_status:references gender:references health_service_reference:string id_card_number:string passport_number:string residency_number:string social_security_number:string telephone_number_1:string telephone_number_2:string telephone_number_3:string mobile_number_1:string mobile_number_2:string mobile_number_3:string fax_number_1:string fax_number_2:string fax_number_3:string email_1:string email_2:string email_3:string url_1:string url_2:string url_3:string notes:text

bundle exec rails generate controller Operations index

---

git-svn notes
=============
Import project from subversion repository:
git svn clone --trunk=trunk/core_models --tags=tags/core_models --branches=branches/core_models --authors-file=/home/everard/.git-authors https://dev.htcl.eu/project/repo/website/rails_engines core_models

New branch linked to subversion repository:
git svn branch -m "Create feature branch" <feature_branch>
git checkout --track -b <feature_branch> <feature_branch>

Integration:
git merge <feature_branch>
- OR -
git merge --squash <feature_branch>

Get latest changes from subversion repository:
git svn rebase

Commit to subversion repository:
git svn rebase && git svn dcommit

Synchronising branches:

Integrate 'wip' into 'development':
git stash save && git checkout wip && git rebase development && git checkout development && git merge wip && git checkout wip && git rebase development && git stash pop

Integrate 'development' into 'master':
git stash save && git checkout master && git svn rebase && git checkout development && git rebase master && git checkout master && git merge development && git svn dcommit && git checkout development && git rebase master && git stash pop

Re-base 'wip' from 'development':
git stash save && git checkout wip && git rebase development && git stash pop

Re-base 'development' from 'master':
git stash save && git checkout master && git svn rebase && git checkout development && git rebase master && git stash pop

Sync 'master' to subversion
git stash save && git checkout master && git svn dcommit && git stash pop

Sync to all shared repositories:
git gc && for branch in master development ; do [[ `git rev-parse --verify --quiet $branch` ]] && (git stash save 'repo-sync' && git pull --rebase gitlab $branch && ([[ `git stash list | grep 'repo-sync'` ]] && git stash pop) ; (for repo in origin tfs gitlab ; do git push -u $repo $branch ; done)) ; done


--
This project rocks and is copyrighted.