require 'geocoder'

module CoreModels
  class Base::Address < ApplicationRecord
    belongs_to :owner,   :polymorphic => true, :optional => true
    belongs_to :city,    :class_name => '::CoreModels::Base::City', :optional => true
    belongs_to :country, :class_name => '::CoreModels::I18n::Country'

    has_many :telephone_numbers, :class_name => '::CoreModels::Base::TelephoneNumber', :as => :owner
    accepts_nested_attributes_for :telephone_numbers, reject_if: :all_blank, allow_destroy: true

    has_many :email_addresses, :class_name => '::CoreModels::Base::EmailAddress', :as => :owner
    accepts_nested_attributes_for :email_addresses, reject_if: :all_blank, allow_destroy: true

    has_many :web_addresses, :class_name => '::CoreModels::Base::WebAddress', :as => :owner
    accepts_nested_attributes_for :web_addresses, reject_if: :all_blank, allow_destroy: true

    #::Gmaps4rails::ActsAsGmappable::ClassMethods.acts_as_gmappable :validation => false
    #acts_as_gmappable :validation => false unless Rails.env == 'test'
    geocoded_by         :gmaps4rails_address                # can also be an IP address
    reverse_geocoded_by :latitude, :longitude
    after_validation    :geocode unless (Rails.env == 'test' || !(ENV['CoreModels_DISABLE_GEOCODING'] && ENV['CoreModels_DISABLE_GEOCODING'].empty?))  # auto-fetch coordinates

    #attr_accessible :room, :building, :address_line_1, :address_line_2, :postal_code, :town, :city, :region
    #attr_accessible :email_1, :email_2, :email_3, :url_1, :url_2, :url_3
    #attr_accessible :telephone_number_1, :telephone_number_2, :telephone_number_3
    #attr_accessible :fax_number_1, :fax_number_2

    #attr_accessible :latitude, :longitude

    #if ::Rails.env.test?
    #attr_accessible :country, :country_id
    #attr_accessible :owner, :owner_id
    #end

    #attr_accessible :world_coordinates

    validates :address_line_1, :presence => true
    # validates :town, :presence => true
    # validates :city, :presence => true
    # validates :region, :presence => true
    # validates :postal_code, :presence => true
    # validates :country, :presence => true

    validate  :city_and_country

    #----------------------------------------------------------------------------

    def to_s
      summary
    end

    def summary
      summary_en
    end

    def summary_en
      output = self.building.blank? ? '' : self.building
      output += self.building.blank? ? self.address_line_1 : ', ' + self.address_line_1
      output += ', ' + self.address_line_2 unless self.address_line_2.blank?
      #output += ', ' + self.town.best_name_value unless self.town.blank?
      output += ', ' + self.town unless self.town.blank?
      output += ', ' + self.city.name unless self.city.blank?
      #output += ', ' + self.region.best_name_value unless self.region.blank?
      output += ', ' + self.region unless self.region.blank?
      output += ', ' + self.country.best_name unless self.country.blank?
      output
    end

    # Formatted addess string suitable for Google Maps query
    def gmaps4rails_address
      output = self.building.blank? ? '' : self.building
      output += self.building.blank? ? self.address_line_1 : ', ' + self.address_line_1
      output += ', ' + self.address_line_2 unless self.address_line_2.blank?
      #output += ', ' + self.town.best_name_value('en') unless self.town.blank?
      output += ', ' + self.town unless self.town.blank?
      output += ', ' + self.city.name unless self.city.blank?
      #output += ', ' + self.region.best_name_value('en') unless self.region.blank?
      output += ', ' + self.region unless self.region.blank?
      output += ', ' + self.postal_code unless self.postal_code.blank?
      output += ', ' + self.country.best_name unless self.country.blank?
      output
    end

    #----------------------------------------------------------------------------

    private

    def city_and_country
      # Validate one or both of city and country are provided
      errors.add('At least one of city or country must be provided') if city.blank? && country.blank?

      # Inject country, if only the city is provided
      self.country = city.country if !city.blank? && country.blank?

      # Validate referential integrity between city/country
      errors.add(:city, :invalid, message: "does not belong to the country '#{country.best_name}'") if !city.blank? && !country.blank? && !city.country == country
    end
  end
end
