# frozen_string_literal: true

module CoreModels
  class PeopleAndCompanies::Company < ApplicationRecord
    belongs_to :head_office_company,   polymorphic: true, optional: true # , :class_name => '::CoreModels::PeopleAndCompanies::Company'
    belongs_to :parent_office_company, polymorphic: true, optional: true # , :class_name => '::CoreModels::PeopleAndCompanies::Company'

    belongs_to :registration_country, class_name: '::CoreModels::I18n::Country', optional: true

    belongs_to :postal_address, class_name: '::CoreModels::Base::Address', optional: true
    belongs_to :registered_address, class_name: '::CoreModels::Base::Address', optional: true
    belongs_to :billing_address, class_name: '::CoreModels::Base::Address', optional: true

    has_one :company_logo, class_name: '::CoreModels::PeopleAndCompanies::CompanyLogo'
    accepts_nested_attributes_for :company_logo

    has_one :logo, through: :company_logo, class_name: '::AssetManagement::AssetData', dependent: :destroy
    accepts_nested_attributes_for :logo

    has_many :address_associations, class_name: '::CoreModels::Base::AddressAssociation', as: :owner
    accepts_nested_attributes_for :address_associations, reject_if: :all_blank, allow_destroy: true

    has_many :addresses, class_name: '::CoreModels::Base::Address', through: :address_associations, as: :owner
    accepts_nested_attributes_for :addresses, reject_if: :all_blank, allow_destroy: true

    has_many :telephone_numbers, class_name: '::CoreModels::Base::TelephoneNumber', as: :owner
    accepts_nested_attributes_for :telephone_numbers, reject_if: :all_blank, allow_destroy: true

    has_many :email_addresses, class_name: '::CoreModels::Base::EmailAddress', as: :owner
    accepts_nested_attributes_for :email_addresses, reject_if: :all_blank, allow_destroy: true

    has_many :web_addresses, class_name: '::CoreModels::Base::WebAddress', as: :owner
    accepts_nested_attributes_for :web_addresses, reject_if: :all_blank, allow_destroy: true

    has_many :subsidiary_offices_companies, class_name: '::CoreModels::PeopleAndCompanies::Company', as: :head_office_company
    accepts_nested_attributes_for :subsidiary_offices_companies, reject_if: :all_blank, allow_destroy: true

    has_many :child_offices_companies, class_name: '::CoreModels::PeopleAndCompanies::Company', as: :parent_office_company
    accepts_nested_attributes_for :child_offices_companies, reject_if: :all_blank, allow_destroy: true

    has_many :departments, class_name: '::CoreModels::PeopleAndCompanies::Department', as: :parent_company
    accepts_nested_attributes_for :departments, reject_if: :all_blank, allow_destroy: true

    # attr_accessible :name, :fiscal_name, :branch_name, :notes, :registration_number, :vat_registration_number
    # attr_accessible :telephone_number_1, :telephone_number_2, :telephone_number_3, :fax_number_1, :fax_number_2, :fax_number_3
    # attr_accessible :email_1, :email_2, :email_3, :url_1, :url_2, :url_3
    # attr_accessible :billing_contact, :billing_email_address

    # if ::Rails.env.test?
    # attr_accessible :head_office_company, :head_office_company_id
    # attr_accessible :parent_office_company, :parent_office_company_id
    # attr_accessible :postal_address, :postal_address_id
    # attr_accessible :billing_address, :billing_address_id
    # attr_accessible :registered_address, :registered_address_id
    # attr_accessible :registration_country, :registration_country_id
    # end

    # attr_accessible :company_logo_attributes

    validates :name, presence: true

    #----------------------------------------------------------------------------

    # Handlers for polymorphic values (using Global ID)

    # Use :to_global_id to populate forms
    def head_office_company_gid
      head_office_company&.to_global_id
    end

    # Set the attribute from a Global ID (handles the form submission)
    def head_office_company_gid=(gid)
      self.head_office_company = GlobalID::Locator.locate gid
    end

    # Use :to_global_id to populate forms
    def parent_office_company_gid
      parent_office_company&.to_global_id
    end

    # Set the attribute from a Global ID (handles the form submission)
    def parent_office_company_gid=(gid)
      self.parent_office_company = GlobalID::Locator.locate gid
    end

    #----------------------------------------------------------------------------

    def to_s
      name
    end

    def display_name
      to_s
    end
  end
end
