module CoreModels
  class PeopleAndCompanies::Department < ApplicationRecord
    belongs_to :parent_company, :class_name => '::CoreModels::PeopleAndCompanies::Company'
    belongs_to :parent_department, :class_name => '::CoreModels::PeopleAndCompanies::Department', optional: true

    has_many   :job_titles, :class_name => '::CoreModels::PeopleAndCompanies::JobTitle'
    accepts_nested_attributes_for :job_titles

    #----------------------------------------------------------------------------

    validates  :name, presence: true, uniqueness: true

    #----------------------------------------------------------------------------

    # Handlers for polymorphic values (using Global ID)

    # Use :to_global_id to populate forms
    def department_owner_gid
      department_owner&.to_global_id
    end

    # Set the attribute from a Global ID (handles the form submission)
    def department_owner_gid=(gid)
      self.department_owner = GlobalID::Locator.locate gid
    end

    #----------------------------------------------------------------------------

    def to_s
      name
    end
  end
end
