module CoreModels
  class PeopleAndCompanies::Person < ApplicationRecord
    belongs_to :home_address, class_name: '::CoreModels::Base::Address', optional: true
    belongs_to :postal_address, class_name: '::CoreModels::Base::Address', optional: true
    belongs_to :contact_address, class_name: '::CoreModels::Base::Address', optional: true
    accepts_nested_attributes_for :home_address
    accepts_nested_attributes_for :postal_address
    accepts_nested_attributes_for :contact_address

    belongs_to :maritial_status, class_name: '::CoreModels::Base::MaritialStatus', optional: true
    belongs_to :gender, class_name: '::CoreModels::Base::Gender', optional: true

    # has_and_belongs_to_many :photos
    has_one :person_photo, class_name: '::CoreModels::PeopleAndCompanies::PersonPhoto'
    has_one :photo, through: :person_photo, class_name: '::AssetManagement::AssetData', dependent: :destroy
    accepts_nested_attributes_for :person_photo
    accepts_nested_attributes_for :photo

    has_many :address_associations, class_name: '::CoreModels::Base::AddressAssociation', as: :owner
    accepts_nested_attributes_for :address_associations, reject_if: :all_blank, allow_destroy: true

    has_many :addresses, class_name: '::CoreModels::Base::Address', through: :address_associations, as: :owner
    accepts_nested_attributes_for :addresses, reject_if: :all_blank, allow_destroy: true

    has_many :telephone_numbers, class_name: '::CoreModels::Base::TelephoneNumber', as: :owner
    accepts_nested_attributes_for :telephone_numbers, reject_if: :all_blank, allow_destroy: true

    has_many :email_addresses, class_name: '::CoreModels::Base::EmailAddress', as: :owner
    accepts_nested_attributes_for :email_addresses, reject_if: :all_blank, allow_destroy: true

    has_many :web_addresses, class_name: '::CoreModels::Base::WebAddress', as: :owner
    accepts_nested_attributes_for :web_addresses, reject_if: :all_blank, allow_destroy: true

    # -----

    belongs_to :company, class_name: '::CoreModels::PeopleAndCompanies::Company', optional: true

    has_many :department_members, :class_name => '::CoreModels::PeopleAndCompanies::DepartmentMember', :foreign_key => :member_id, dependent: :destroy
    accepts_nested_attributes_for :department_members

    has_many :department_memberships, :class_name => '::CoreModels::PeopleAndCompanies::Department', :through => :department_members, :source => :department
    accepts_nested_attributes_for :department_memberships

    # -----

    # attr_accessible :title, :forename, :middle_names, :surname, :birthdate
    # attr_accessible :email_1, :email_2, :email_3, :url_1, :url_2, :url_3
    # attr_accessible :telephone_number_1, :telephone_number_2, :telephone_number_3
    # attr_accessible :mobile_number_1, :mobile_number_2, :mobile_number_3
    # attr_accessible :fax_number_1, :fax_number_2, :fax_number_3
    # attr_accessible :health_service_reference, :id_card_number, :passport_number, :residency_number, :social_security_number
    # attr_accessible :notes

    # if ::Rails.env.test?
    # attr_accessible :home_address, :home_address_id
    # attr_accessible :postal_address, :postal_address_id
    # attr_accessible :contact_address, :contact_address_id
    # attr_accessible :maritial_status, :maritial_status_id
    # attr_accessible :gender, :gender_id
    # end

    # attr_accessible :person_photo_attributes

    validate :has_forename_or_surname

    #----------------------------------------------------------------------------

    def to_s
      display_name
    end

    def display_name
      name = ''
      name += "#{title} " unless title.blank? || surname.blank?
      name += "#{forename} " unless forename.blank?
      name += surname.to_s unless surname.blank?
      name
    end

    def age
      if birthdate.nil?
        0
      else
        now = Time.now.utc.to_date
        now.year - birthdate.year - (now.month > birthdate.month || (now.month == birthdate.month && birthdate.day >= birthdate.day) ? 0 : 1)
      end
    end

    def best_home_address
      if !home_address.nil?
        home_address
      elsif address_associations.where(address_type: ::CoreModels::Base::AddressType.named('Home')).size == 1
        address_associations.where(address_type: ::CoreModels::Base::AddressType.named('Home')).first.address
      end
    end

    def best_postal_address
      if !postal_address.nil?
        postal_address
      elsif address_associations.where(address_type: ::CoreModels::Base::AddressType.named('Postal')).size == 1
        address_associations.where(address_type: ::CoreModels::Base::AddressType.named('Postal')).first.address
      end
    end

    def best_contact_address
      if !contact_address.nil?
        contact_address
      elsif address_associations.where(address_type: ::CoreModels::Base::AddressType.named('Contact')).size == 1
        address_associations.where(address_type: ::CoreModels::Base::AddressType.named('Contact')).first.address
      end
    end

    #----------------------------------------------------------------------------

    private

    def has_forename_or_surname
      errors[:base] << 'must have a forename and/or a surname' if forename.blank? && surname.blank?
    end
  end
end
